//----------------------------------------------------------
// CREVBUF.CPP - EAX Buffer Property Set Interface Class Definition
//               (the Buffer)
//
// Example code which makes it easy to use the EAX
// buffer property set interface
//
//----------------------------------------------------------
#include "crevbuf.h"

//----------------------------------------------------------
// CReverbBuffer(LPDIRECTSOUND3DBUFFER p3DBuf)
//
// DESCRITPTION: Constructor
//               Creates a new CReverbBuffer object based on
//               a DirectSound3DBuffer interface.
//
// PARAMETERS: p3DBuf - pointer to a 3D buffer interface
//
// RETURNS: no return value, check with PropertySetOk member
//          function to check if object was correctly created 
//
//----------------------------------------------------------
CReverbBuffer::CReverbBuffer(LPDIRECTSOUND3DBUFFER p3DBuf)
   {                                    // NOTE: Use with PropertySetOk member
   if (FAILED(p3DBuf->QueryInterface(IID_IKsPropertySet,(void**)&pPropertySet)))
      {                                 // Get property set interface
      pPropertySet=NULL;
      return;
      }
   if (!QueryProperty(DSPROPERTY_EAXBUFFER_ALL)   // Check the properties
      || !QueryProperty(DSPROPERTY_EAXBUFFER_REVERBMIX))
      pPropertySet=NULL;
   }

//----------------------------------------------------------
// int PropertySetOk()
//
// DESCRITPTION: Checks to see if a valid property set interface
//               was created
//
// PARAMETERS: none
//
// RETURNS: TRUE if valid property set interface.
//
//----------------------------------------------------------
int CReverbBuffer::PropertySetOk()
   {
   return pPropertySet != NULL;         // Checks to see if interface is valid
   }

//----------------------------------------------------------
// int QueryProperty(unsigned long propertyId)
//
// DESCRITPTION: Checks to see if the property set interface
//               supports a particular property
//
// PARAMETERS: property ID
//
// RETURNS: TRUE if the property is supported.
//
//----------------------------------------------------------
int CReverbBuffer::QueryProperty(unsigned long propertyId)
   {
   unsigned long support;               // variable to hold support status

   support=0;                           // Query the property
   if (FAILED(pPropertySet->QuerySupport(DSPROPSETID_EAXBUFFER_ReverbProperties,
                                         propertyId,&support))
      || (support & CREVERBBUFFER_SETGET) != CREVERBBUFFER_SETGET)
      return FALSE;
   return TRUE;
   }

//----------------------------------------------------------
// void SetReverbMix(float mix)
//
// DESCRITPTION: Changes the reverb mix value
//
// PARAMETERS: mix value
//
// RETURNS: no return value.
//
//----------------------------------------------------------
void CReverbBuffer::SetReverbMix(float mix)
   {
   pPropertySet->Set(DSPROPSETID_EAXBUFFER_ReverbProperties,
                     DSPROPERTY_EAXBUFFER_REVERBMIX,NULL,0,
                     &mix,sizeof(float));
   }

//----------------------------------------------------------
// void SetAll(EAXBUFFER_REVERBPROPERTIES *pProperties)
//
// DESCRITPTION: Sets all reverb buffer properties.
//
// PARAMETERS: pointer to properties structure
//
// RETURNS: no return value.
//
//----------------------------------------------------------
void CReverbBuffer::SetAll(EAXBUFFER_REVERBPROPERTIES *pProperties)
   {
   pPropertySet->Set(DSPROPSETID_EAXBUFFER_ReverbProperties,
                     DSPROPERTY_EAXBUFFER_ALL,NULL,0,
                     pProperties,sizeof(EAXBUFFER_REVERBPROPERTIES));
   }


//----------------------------------------------------------
// float GetReverbMix()
//
// DESCRITPTION: Gets the current reverb mix
//
// PARAMETERS: none
//
// RETURNS: mix value
//
//----------------------------------------------------------
float CReverbBuffer::GetReverbMix()
   {
   float mix=CREVERBBUFFER_INVALID_VALUE;
   unsigned long r;

   pPropertySet->Get(DSPROPSETID_EAXBUFFER_ReverbProperties,
                     DSPROPERTY_EAXBUFFER_REVERBMIX,NULL,0,
                     &mix,sizeof(float),&r);
   return mix;
   }

//----------------------------------------------------------
// void GetAll(EAXBUFFER_REVERBPROPERTIES *pProperties)
//
// DESCRITPTION: Gets all the reverb buffer properties
//
// PARAMETERS: none
//
// RETURNS: structure containing reverb settings
//
//----------------------------------------------------------
void CReverbBuffer::GetAll(EAXBUFFER_REVERBPROPERTIES *pProperties)
   {
   unsigned long r;

   pPropertySet->Get(DSPROPSETID_EAXBUFFER_ReverbProperties,
                     DSPROPERTY_EAXBUFFER_ALL,NULL,0,
                     pProperties,sizeof(EAXBUFFER_REVERBPROPERTIES),&r);
   }
