//----------------------------------------------------------
// CREVERB.CPP - EAX Property Set Interface Class Definition
//               (the listener)
//
// Example code which makes it easy to use the EAX
// property set interface
//
//----------------------------------------------------------
#include <objbase.h>
#include <initguid.h>         // include this once to instantiate EAX GUIDs
#include "creverb.h"

//----------------------------------------------------------
// CReverb(LPDIRECTSOUND3DBUFFER p3DBuf)
//
// DESCRITPTION: Constructor
//               Creates a new CReverb object based on
//               a DirectSound3DBuffer interface.  Any valid
//               3D buffer will do.
//
// PARAMETERS: p3DBuf - pointer to a 3D buffer interface
//
// RETURNS: no return value, check with PropertySetOk member
//          function to check if object was correctly created 
//
//----------------------------------------------------------
CReverb::CReverb(LPDIRECTSOUND3DBUFFER p3DBuf)
   {                                    // NOTE: Use with PropertySetOk member
   if (FAILED(p3DBuf->QueryInterface(IID_IKsPropertySet,(void**)&pPropertySet)))
      {                                 // Get property set interface
      pPropertySet=NULL;
      return;
      }
   if (!QueryProperty(DSPROPERTY_EAX_ALL)   // Check the properties
      || !QueryProperty(DSPROPERTY_EAX_ENVIRONMENT)
      || !QueryProperty(DSPROPERTY_EAX_VOLUME)
      || !QueryProperty(DSPROPERTY_EAX_DECAYTIME)
      || !QueryProperty(DSPROPERTY_EAX_DAMPING))
      pPropertySet=NULL;
   }

//----------------------------------------------------------
// int PropertySetOk()
//
// DESCRITPTION: Checks to see if a valid property set interface
//               was created
//
// PARAMETERS: none
//
// RETURNS: TRUE if valid property set interface.
//
//----------------------------------------------------------
int CReverb::PropertySetOk()
   {
   return pPropertySet != NULL;         // Checks to see if interface is valid
   }

//----------------------------------------------------------
// int QueryProperty(unsigned long propertyId)
//
// DESCRITPTION: Checks to see if the property set interface
//               supports a particular property
//
// PARAMETERS: property ID
//
// RETURNS: TRUE if the property is supported.
//
//----------------------------------------------------------
int CReverb::QueryProperty(unsigned long propertyId)
   {
   unsigned long support;               // variable to hold support status

   support=0;                           // Query the property
   if (FAILED(pPropertySet->QuerySupport(DSPROPSETID_EAX_ReverbProperties,
                                         propertyId,&support))
      || (support & CREVERB_SETGET) != CREVERB_SETGET)
      return FALSE;
   return TRUE;
   }

//----------------------------------------------------------
// void SetEnvironment(unsigned long envId)
//
// DESCRITPTION: Selects a new environment, from the list
//               in EAX.H.  e.g. EAX_ENVIRONMENT_CAVE
//
// PARAMETERS: Environment ID
//
// RETURNS: no return value.
//
//----------------------------------------------------------
void CReverb::SetEnvironment(unsigned long envId)
   {
   pPropertySet->Set(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_ENVIRONMENT,NULL,0,
                     &envId,sizeof(unsigned long));
   }

//----------------------------------------------------------
// void SetVolume(float volume)
//
// DESCRITPTION: Changes the reverb volume
//
// PARAMETERS: volume value
//
// RETURNS: no return value.
//
//----------------------------------------------------------
void CReverb::SetVolume(float volume)
   {
   pPropertySet->Set(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_VOLUME,NULL,0,
                     &volume,sizeof(float));
   }

//----------------------------------------------------------
// void SetDecayTime(float time)
//
// DESCRITPTION: Changes the reverb decay time
//
// PARAMETERS: decay time value
//
// RETURNS: no return value.
//
//----------------------------------------------------------
void CReverb::SetDecayTime(float time)
   {
   pPropertySet->Set(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_DECAYTIME,NULL,0,
                     &time,sizeof(float));
   }

//----------------------------------------------------------
// void SetDamping(float damping)
//
// DESCRITPTION: Changes the reverb damping
//
// PARAMETERS: damping value
//
// RETURNS: no return value.
//
//----------------------------------------------------------
void CReverb::SetDamping(float damping)
   {
   pPropertySet->Set(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_DAMPING,NULL,0,
                     &damping,sizeof(float));
   }

//----------------------------------------------------------
// void SetAll(EAX_REVERBPROPERTIES *pProperties)
//
// DESCRITPTION: Sets up an entire reverb environment including
//               standard properties.
//
// PARAMETERS: pointer to properties structure
//
// RETURNS: no return value.
//
//----------------------------------------------------------
void CReverb::SetAll(EAX_REVERBPROPERTIES *pProperties)
   {
   pPropertySet->Set(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_ALL,NULL,0,
                     pProperties,sizeof(EAX_REVERBPROPERTIES));
   }

//----------------------------------------------------------
// void SetPreset(unsigned long envId, float volume, float time, float damping)
//
// DESCRITPTION: Sets up an entire reverb environment including
//               standard properties, by way of a preset.
//
// PARAMETERS: individual properties
//
// RETURNS: no return value.
//
//----------------------------------------------------------
void CReverb::SetPreset(unsigned long envId, float volume,
                            float time, float damping)
   {
   EAX_REVERBPROPERTIES props={envId,volume,time,damping};
   
   pPropertySet->Set(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_ALL,NULL,0,
                     &props,sizeof(EAX_REVERBPROPERTIES));
   }

//----------------------------------------------------------
// unsigned long GetEnvironment()
//
// DESCRITPTION: Gets the current reverb environment
//
// PARAMETERS: none
//
// RETURNS: Environment ID
//
//----------------------------------------------------------
unsigned long CReverb::GetEnvironment()
   {
   unsigned long envId=EAX_ENVIRONMENT_GENERIC;
   unsigned long r;

   pPropertySet->Get(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_ENVIRONMENT,NULL,0,
                     &envId,sizeof(unsigned long),&r);
   return envId;
   }

//----------------------------------------------------------
// float GetVolume()
//
// DESCRITPTION: Gets the current reverb volume
//
// PARAMETERS: none
//
// RETURNS: volume value
//
//----------------------------------------------------------
float CReverb::GetVolume()
   {
   float volume=CREVERB_INVALID_VALUE;
   unsigned long r;

   pPropertySet->Get(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_VOLUME,NULL,0,
                     &volume,sizeof(float),&r);
   return volume;
   }

//----------------------------------------------------------
// float GetDecayTime()
//
// DESCRITPTION: Gets the current reverb decay time
//
// PARAMETERS: none
//
// RETURNS: decay time value
//
//----------------------------------------------------------
float CReverb::GetDecayTime()
   {
   float time=CREVERB_INVALID_VALUE;
   unsigned long r;

   pPropertySet->Get(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_DECAYTIME,NULL,0,
                     &time,sizeof(float),&r);
   return time;
   }

//----------------------------------------------------------
// float GetDamping()
//
// DESCRITPTION: Gets the current reverb damping
//
// PARAMETERS: none
//
// RETURNS: damping value
//
//----------------------------------------------------------
float CReverb::GetDamping()
   {
   float damping=CREVERB_INVALID_VALUE;
   unsigned long r;

   pPropertySet->Get(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_DAMPING,NULL,0,
                     &damping,sizeof(float),&r);
   return damping;
   }

//----------------------------------------------------------
// void GetAll(EAX_REVERBPROPERTIES *pProperties)
//
// DESCRITPTION: Gets all the current reverb settings, including
//               environment and standard properties.
//
// PARAMETERS: none
//
// RETURNS: structure containing reverb settings
//
//----------------------------------------------------------
void CReverb::GetAll(EAX_REVERBPROPERTIES *pProperties)
   {
   unsigned long r;

   pPropertySet->Get(DSPROPSETID_EAX_ReverbProperties,
                     DSPROPERTY_EAX_ALL,NULL,0,
                     pProperties,sizeof(EAX_REVERBPROPERTIES),&r);
   }
